package org.artifactory.lock.service;

import org.artifactory.common.ConstantValues;
import org.artifactory.concurrent.NonStandardLockCategory;
import org.artifactory.lock.util.ReleaseLockStrategyUtils;
import org.artifactory.storage.db.locks.service.DbLocksService;
import org.artifactory.storage.db.servers.service.ArtifactoryServersCommonService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.Set;
import java.util.stream.Collectors;

import static org.artifactory.common.ConstantValues.haHeartbeatStaleIntervalSecs;

/**
 * @author Mor Merhav
 */
@Component
public class StandardReleaseLockStrategy implements ReleaseLockStrategy {

    private final DbLocksService dbLocksService;
    private final ArtifactoryServersCommonService serversCommonService;

    @Autowired
    public StandardReleaseLockStrategy(DbLocksService dbLocksService,
            ArtifactoryServersCommonService serversCommonService) {
        this.dbLocksService = dbLocksService;
        this.serversCommonService = serversCommonService;
    }

    public static int getStdOrphanLockGraceTimeInSeconds() {
        return haHeartbeatStaleIntervalSecs.getInt() * 3;
    }

    @Override
    public void releaseOrphaned() {
        Set<String> activeServers = ReleaseLockStrategyUtils.findActiveServers(serversCommonService,
                getStdOrphanLockGraceTimeInSeconds());
        Set<String> nonStandardCategories = Arrays.stream(NonStandardLockCategory.values())
                .map(NonStandardLockCategory::getName)
                .collect(Collectors.toSet());
        dbLocksService.releaseOrphanLocksExcludingServersAndCategories(activeServers, nonStandardCategories);
    }

    @Override
    public void releaseExpired() {
        Set<String> nonStandardCategories = Arrays.stream(NonStandardLockCategory.values())
                .map(NonStandardLockCategory::getName)
                .collect(Collectors.toSet());
        dbLocksService.releaseExpiredLocksExcludingCategories(nonStandardCategories,
                ConstantValues.hazelcastMaxLockLeaseTime.getLong());
    }
}
