package org.artifactory.maven;

import org.artifactory.addon.ArtifactoryRunningMode;
import org.artifactory.descriptor.repo.RepoLayout;
import org.artifactory.descriptor.repo.RepoLayoutBuilder;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.util.RepoLayoutUtils;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import static org.testng.Assert.assertEquals;

@Test
public class MavenMetadataServiceImplTest {

    private MavenMetadataServiceImpl service = new MavenMetadataServiceImpl();

    @Test(dataProvider = "skipCalculationDataProvider")
    public void TestSkipCalculation(RepoType type, RepoLayout repoLayout, ArtifactoryRunningMode runningModel,
            boolean expected) {
        assertEquals(service.skipCalculation(type, repoLayout, runningModel), expected,
                "Layout = " + repoLayout.getName());
    }

    @DataProvider(name = "skipCalculationDataProvider")
    public Object[][] skipCalculationDataProvider() {
        return new Object[][] {
                // For backward comparability Maven indexing is supported on repositories with the right layout
                // even if they are not Maven family
                {RepoType.Generic, RepoLayoutUtils.MAVEN_2_DEFAULT, ArtifactoryRunningMode.OSS, false},
                {RepoType.Generic, RepoLayoutUtils.MAVEN_1_DEFAULT, ArtifactoryRunningMode.OSS, true},
                {RepoType.Gradle, RepoLayoutUtils.MAVEN_1_DEFAULT, ArtifactoryRunningMode.OSS, false},

                // Repos with null type should always be indexed - as used to be in the old code.
                {null, RepoLayoutUtils.MAVEN_2_DEFAULT, ArtifactoryRunningMode.PRO, false},
                {null, RepoLayoutUtils.MAVEN_1_DEFAULT, ArtifactoryRunningMode.PRO, false},
                {null, RepoLayoutUtils.MAVEN_2_DEFAULT, ArtifactoryRunningMode.JCR, false},

                // Maven family should never be skipped - even at wrong layout
                {RepoType.Ivy,
                        new RepoLayoutBuilder()
                                .name("FakeLayout").build(),
                        ArtifactoryRunningMode.PRO, false},

                // Maven 2 layout should be ignored if the license is not right and the repo is not Maven family
                {RepoType.Generic, RepoLayoutUtils.MAVEN_2_DEFAULT, ArtifactoryRunningMode.JCR, true},
                {RepoType.Generic, RepoLayoutUtils.MAVEN_2_DEFAULT, ArtifactoryRunningMode.AOL_JCR, true},
                {RepoType.Generic, RepoLayoutUtils.MAVEN_2_DEFAULT, ArtifactoryRunningMode.CONAN, true},

                // Maven family should never be skipped - even at wrong license
                {RepoType.Maven, RepoLayoutUtils.MAVEN_2_DEFAULT, ArtifactoryRunningMode.JCR, false}
        };
    }
}
