package org.artifactory.request;

import org.artifactory.security.access.AccessTokenAuthentication;
import org.jfrog.access.token.JwtAccessToken;
import org.mockito.Mockito;
import org.testng.Assert;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 * @author Shay Bagants
 */
@Test
public class RequestResponseHelperTest {

    @Test(dataProvider = "provideUserNames", description = "RTFACT-20177")
    public void testGetUserAgentIfXrayUser(String userName, String agent) {
        Request request = mock(Request.class);
        when(request.getHeader("User-Agent")).thenReturn(agent);
        String result = RequestResponseHelper.getUserAgentIfXrayUser(request, userName, null);
        Assert.assertEquals(result, agent);
    }

    @Test(dataProvider = "provideUserNamesForXrayToken", description = "RTFACT-20177")
    public void testGetUserAgentIfXrayUserUsingAuthenticationToken(String subject, String agent) {
        AccessTokenAuthentication accessTokenAuthentication = Mockito.mock(AccessTokenAuthentication.class);
        JwtAccessToken jwtAccessToken = Mockito.mock(JwtAccessToken.class);
        when(accessTokenAuthentication.getAccessToken()).thenReturn(jwtAccessToken);
        when(jwtAccessToken.getSubject()).thenReturn(subject);
        Request request = mock(Request.class);
        when(request.getHeader("User-Agent")).thenReturn(agent);
        String result = RequestResponseHelper.getUserAgentIfXrayUser(request, "", accessTokenAuthentication);
        Assert.assertEquals(result, agent);
    }

    @DataProvider
    public static Object[][] provideUserNames() {
        return new Object[][] {
                //user,  agent
                {"xray", "xray-agent"},
                {"token:jfrt@123", ""},
                {"token:xyz@123", ""},
                {"token:abc", ""},
                {"xday", ""},
                {"admin", ""},
                {"system", ""},
                {"me", ""}
        };
    }

    @DataProvider
    public static Object[][] provideUserNamesForXrayToken() {
        return new Object[][] {
                //user,  agent
                {"jfxr@123", "xray-token"},
                {"xray", ""},
                {"jfrt@123", ""},
                {"token:xyz@123", ""},
                {"abc", ""},
                {"xday", ""},
                {"admin", ""},
                {"system", ""},
                {"me", ""},
                {"jfxana@123", "xray-token"},
                {"jfxpst@123", "xray-token"},
                {"jfxidx@123", "xray-token"},
        };
    }
}