package org.artifactory.schedule;

import java.util.List;

/**
 * @author Mor Merhav
 */
interface InternalTask extends Task {

    boolean isClusterSingleton();

    void setClusterSingleton(boolean clusterSingleton);

    void schedule(boolean waitForRunning);

    void cancel(boolean wait, long timeout);

    void pause(boolean wait);

    void pause(boolean wait, String stoppingTaskToken);

    /**
     * Stops but does not unschedule the task (can transition back to running state)
     */
    void stop(boolean wait);

    void stop(boolean wait, String stoppingTaskToken);

    boolean resume();

    boolean resume(String stoppingTaskToken);

    void completed();

    /**
     * Wait for the task to stop running
     */
    boolean waitForCompletion(long timeout);

    boolean started();

    void lockState();

    void unlockState();

    boolean isSingleton();

    void setSingleton(boolean singleton);

    boolean isManuallyActivated();

    void setManuallyActivated(boolean manuallyActivated);

    boolean wasCompleted();

    String getTaskLockKeyIncludingKeyAttributes(List<String> keyAttributes);

    boolean waitingForProcess();

    /**
     * Starts or schedules the task
     */
    void scheduleTask();

    /**
     * Stops or unschedules the task
     */
    void cancelTask();
}
