package org.artifactory.storage.db.environment.converter.shared.version.v3;

import lombok.SneakyThrows;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.environment.converter.BasicEnvironmentConverter;
import org.artifactory.storage.db.init.CommonDataSourceFactory;
import org.artifactory.storage.db.util.JdbcHelper;
import org.artifactory.storage.db.version.converter.DBSqlConverter;
import org.artifactory.version.ArtifactoryVersionProvider;
import org.artifactory.version.CompoundVersionDetails;
import org.jfrog.storage.DbType;
import org.jfrog.storage.JFrogDataSource;

/**
 * @author Mor Merhav
 */
public class NullEventCategoryToReplicationConverter implements BasicEnvironmentConverter {
    private static final String CONVERSION_NAME = "v224_change_null_event_category_to_replication";

    @Override
    public boolean isInterested(ArtifactoryHome home, CompoundVersionDetails source, CompoundVersionDetails target) {
        return source.getVersion().before(ArtifactoryVersionProvider.v770.get());
    }

    @SneakyThrows
    @Override
    public void convert(ArtifactoryHome artifactoryHome, CompoundVersionDetails source, CompoundVersionDetails target) {
        DbType dbType = artifactoryHome.getDBProperties().getDbType();
        JFrogDataSource dataSource = CommonDataSourceFactory.createDataSource(artifactoryHome.getDBProperties(), true);
        try (JdbcHelper jdbcHelper = new JdbcHelper(dataSource)) {
            new DBSqlConverter(CONVERSION_NAME).convert(jdbcHelper, dbType);
        }
    }

    @Override
    public void assertConversionPreconditions(ArtifactoryHome home) {
        // noop
    }
}
