package org.artifactory.storage.db.util.itest;

import org.artifactory.storage.db.itest.DbBaseTest;
import org.jfrog.storage.DbType;
import org.jfrog.storage.QueryControls;
import org.jfrog.storage.util.DbUtils;
import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * @author Alexei Vainshtein
 */
public class DbUtilsTest extends DbBaseTest {

    @BeforeClass
    public void setup() {
        importSql("/sql/nodes.sql");
    }

    @Test
    public void testColumnSize() throws SQLException {
        Assert.assertTrue(DbUtils.columnExists(jdbcHelper, dbProperties.getDbType(), "tasks", "task_context"));
        Assert.assertEquals(DbUtils.getColumnSize(jdbcHelper, dbProperties.getDbType(), "tasks", "task_context"), 2048);
    }

    @Test
    public void testGetConnectionWithDefaultTimeout() throws SQLException {
        QueryControls queryControls = new QueryControls.Builder().build();
        assertConnectionResultWithTimeout(queryControls, DbUtils.DEFAULT_TIMEOUT_MILLISECONDS);
    }

    private void assertConnectionResultWithTimeout(QueryControls queryControls, int timeout) throws SQLException {
        ExecutorService executorService = jdbcHelper.getDataSource().getExecutorService();
        try (Connection connection = DbUtils.getConnection(queryControls, jdbcHelper.getDataSource(),
                executorService)) {
            if (dbProperties.getDbType() == DbType.MYSQL) {
                // MySql is setting the timeout using the executor in a different thread.
                // Need to wait for that thread to be done before executing the getNetworkTimeout.
                // Otherwise, the returned result will be the previous one.
                executorService.awaitTermination(1000, TimeUnit.MILLISECONDS);
            }
            int networkTimeout = connection.getNetworkTimeout();
            if (dbProperties.getDbType() == DbType.DERBY) {
                Assert.assertEquals(networkTimeout, 0);
                return;
            }
            Assert.assertEquals(networkTimeout, timeout);
        } catch (InterruptedException e) {
            Assert.fail(e.getMessage());
        }
    }
}
