package org.artifactory.ui.rest.service.admin.configuration.repositories.util;

import org.artifactory.api.repo.RepositoryService;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.descriptor.repo.LocalRepoDescriptor;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.descriptor.repo.VirtualRepoDescriptor;
import org.artifactory.projects.ProjectsService;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.ArtifactoryRestResponse;
import org.artifactory.ui.rest.model.admin.configuration.repository.info.AvailableRepositories;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import java.util.List;

/**
 * @author BarakH
 */
@Test
public class GetAvailableRepositoriesTest {
    private enum UserType {SIMPLE, PROJECT_ADMIN, ADMIN}

    private static final String TEST_PROJECT = "stam";

    @Mock
    private RepositoryService repositoryService;
    @Mock
    private ProjectsService projectsService;
    @Mock
    private AuthorizationService authorizationService;

    private GetAvailableRepositories getAvailableRepositories;

    @BeforeClass
    void init() {
        MockitoAnnotations.initMocks(this);
    }

    @BeforeMethod
    public void setUp() {
        getAvailableRepositories =
                new GetAvailableRepositories(repositoryService, projectsService, authorizationService);
        LocalRepoDescriptor localRepo = new LocalRepoDescriptor();
        localRepo.setKey("A");
        VirtualRepoDescriptor virtualRepo = new VirtualRepoDescriptor();
        virtualRepo.setKey("C");
        Mockito.when(repositoryService.getLocalRepoDescriptors()).thenReturn(List.of(localRepo));
        Mockito.when(repositoryService.getVirtualRepoDescriptors()).thenReturn(List.of(virtualRepo));
    }

    @Test(dataProvider = "provideUserType")
    public void testAvailableRepos(UserType userType, boolean shouldBeExist) {
        ArtifactoryRestRequest restRequest = Mockito.mock(ArtifactoryRestRequest.class);
        ArtifactoryRestResponse response = new ArtifactoryRestResponse();
        Mockito.when(restRequest.getQueryParamByKey("type")).thenReturn(RepoType.Generic.getDisplayName());
        Mockito.when(restRequest.getQueryParamByKey("project")).thenReturn(TEST_PROJECT);
        switch (userType) {
            case SIMPLE:
                Mockito.when(authorizationService.isAdmin()).thenReturn(false);
                Mockito.when(projectsService.getProjectRepositories(TEST_PROJECT)).thenReturn(List.of());
                break;
            case PROJECT_ADMIN:
                Mockito.when(authorizationService.isAdmin()).thenReturn(false);
                Mockito.when(projectsService.getProjectRepositories(TEST_PROJECT)).thenReturn(List.of("A", "C"));
                break;
            case ADMIN:
                Mockito.when(authorizationService.isAdmin()).thenReturn(true);
                Mockito.when(projectsService.getProjectRepositories(TEST_PROJECT)).thenReturn(List.of());
                break;
        }
        getAvailableRepositories.execute(restRequest, response);
        AvailableRepositories availableRepositories = (AvailableRepositories) response.getIModel();
        if (shouldBeExist) {
            Assert.assertTrue(availableRepositories.getAvailableLocalRepos().contains("A"));
            Assert.assertTrue(availableRepositories.getAvailableVirtualRepos().contains("C"));
        } else {
            Assert.assertFalse(availableRepositories.getAvailableLocalRepos().contains("A"));
            Assert.assertFalse(availableRepositories.getAvailableVirtualRepos().contains("C"));
        }
    }

    @DataProvider
    public static Object[][] provideUserType() {
        return new Object[][] {
                //{UserType.SIMPLE, false},
                {UserType.PROJECT_ADMIN, true},
                //{UserType.ADMIN, true},
        };
    }
}