package org.artifactory.ui.rest.service.admin.configuration.repositories.util;

import org.artifactory.api.config.CentralConfigService;
import org.artifactory.api.repo.RepositoryService;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.descriptor.config.CentralConfigDescriptor;
import org.artifactory.descriptor.repo.LocalRepoDescriptor;
import org.artifactory.projects.ProjectsService;
import org.artifactory.repo.RepoDetailsType;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.ArtifactoryRestResponse;
import org.artifactory.ui.rest.model.admin.configuration.repository.info.RepositoryInfo;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import java.util.List;
import java.util.Set;

/**
 * @author BarakH
 */
@Test
public class GetRepositoryInfoServiceTest {
    private enum UserType {SIMPLE, PROJECT_ADMIN, ADMIN}

    @Mock
    private RepositoryService repositoryService;
    @Mock
    private CentralConfigService configService;
    @Mock
    private ProjectsService projectsService;
    @Mock
    private AuthorizationService authorizationService;

    GetRepositoryInfoService getRepositoryInfoService;

    @BeforeClass
    void init() {
        MockitoAnnotations.initMocks(this);
    }

    @BeforeMethod
    public void setUp() {
        getRepositoryInfoService =
                new GetRepositoryInfoService(repositoryService, configService, projectsService, authorizationService);
        LocalRepoDescriptor localRepo = new LocalRepoDescriptor();
        localRepo.setKey("A");
        Mockito.when(repositoryService.getLocalRepoDescriptors()).thenReturn(List.of(localRepo));
        CentralConfigDescriptor descriptor = Mockito.mock(CentralConfigDescriptor.class);
        Mockito.when(descriptor.getMultiLocalReplications("A")).thenReturn(List.of());
        Mockito.when(configService.getDescriptor()).thenReturn(descriptor);
    }

    @Test(dataProvider = "provideUserType")
    public void testGetRepositoriesInfo(UserType userType, boolean shouldBeExist) {
        ArtifactoryRestRequest restRequest = Mockito.mock(ArtifactoryRestRequest.class);
        ArtifactoryRestResponse response = new ArtifactoryRestResponse();
        Mockito.when(restRequest.getPathParamByKey("repoType")).thenReturn(RepoDetailsType.LOCAL_REPO);
        switch (userType) {
            case SIMPLE:
                Mockito.when(authorizationService.isAdmin()).thenReturn(false);
                break;
            case PROJECT_ADMIN:
                Mockito.when(authorizationService.isAdmin()).thenReturn(false);
                Mockito.when(projectsService.getProjectAdminRepositories()).thenReturn(Set.of("A", "C"));
                break;
            case ADMIN:
                Mockito.when(authorizationService.isAdmin()).thenReturn(true);
                break;
        }
        getRepositoryInfoService.execute(restRequest, response);
        List<RepositoryInfo> result = (List<RepositoryInfo>) response.getiModelList();
        if (shouldBeExist) {
            Assert.assertEquals("A", result.get(0).getRepoKey());
        } else {
            Assert.assertTrue(result.isEmpty());
        }
    }

    @DataProvider
    public static Object[][] provideUserType() {
        return new Object[][] {
                {UserType.SIMPLE, false},
                {UserType.PROJECT_ADMIN, true},
                {UserType.ADMIN, true},
        };
    }
}