package org.artifactory.ui.rest.service.setmeup;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.CoreAddons;
import org.artifactory.aol.AolService;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.request.ArtifactoryRequest;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.artifactory.ui.rest.model.setmeup.SetMeUpModel;
import org.artifactory.ui.rest.model.utils.repositories.RepoKeyType;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 * @author Alexei Vainshtein
 */
public class GetSetMeUpServiceTest extends ArtifactoryHomeBoundTest {

    // Required for GetSetMeUpService injection
    @Mock
    private CentralConfigService centralConfigService;

    @InjectMocks
    private GetSetMeUpService setMeUpService;

    @BeforeClass
    private void setup() {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    public void testSetMeUpModelWithoutFlag() {
        setupMocks(false);
        try {
            HttpServletRequest servletRequest = mock(HttpServletRequest.class);
            when(servletRequest.getHeader(ArtifactoryRequest.JFROG_OVERRIDE_BASE_URL))
                    .thenReturn("https://test.jfrog.ui/");
            ArtifactoryRestRequest request = mock(ArtifactoryRestRequest.class);
            when(request.getServletRequest()).thenReturn(servletRequest);
            List<RepoKeyType> repoKeyTypes = new ArrayList<>();
            SetMeUpModel setMeUpModel = setMeUpService.getSetMeUpModel(request, repoKeyTypes);
            Assert.assertFalse(setMeUpModel.isSkipDockerDns());
        } finally {
            ArtifactoryContextThreadBinder.unbind();
        }

    }

    @Test
    public void testSetMeUpModelWithFlag() {
        setupMocks(true);
        try {
            HttpServletRequest servletRequest = mock(HttpServletRequest.class);
            when(servletRequest.getHeader(ArtifactoryRequest.JFROG_OVERRIDE_BASE_URL))
                    .thenReturn("https://test.jfrog.ui/");
            ArtifactoryRestRequest request = mock(ArtifactoryRestRequest.class);
            when(request.getServletRequest()).thenReturn(servletRequest);
            List<RepoKeyType> repoKeyTypes = new ArrayList<>();
            SetMeUpModel setMeUpModel = setMeUpService.getSetMeUpModel(request, repoKeyTypes);
            Assert.assertTrue(setMeUpModel.isSkipDockerDns());
        } finally {
            ArtifactoryContextThreadBinder.unbind();
        }
    }

    private void setupMocks(boolean isSkipDns) {
        ArtifactoryContext artifactoryContext = mock(ArtifactoryContext.class);
        AddonsManager addonsManager = mock(AddonsManager.class);
        CoreAddons coreAddons = mock(CoreAddons.class);
        AolService aolService = mock(AolService.class);
        when(aolService.skipDockerDns()).thenReturn(isSkipDns);
        when(coreAddons.getArtifactoryServerName()).thenReturn("jfrog");
        when(addonsManager.addonByType(CoreAddons.class)).thenReturn(coreAddons);
        when(artifactoryContext.beanForType(AddonsManager.class)).thenReturn(addonsManager);
        when(artifactoryContext.beanForType(AolService.class)).thenReturn(aolService);
        CentralConfigService centralConfigService = mock(CentralConfigService.class);
        when(centralConfigService.getListeningPort()).thenReturn(-1);
        when(artifactoryContext.beanForType(CentralConfigService.class)).thenReturn(centralConfigService);
        ArtifactoryContextThreadBinder.bind(artifactoryContext);
    }
}