package org.artifactory.rest.resource.system;

import org.apache.http.HttpStatus;
import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.license.ArtifactoryBaseLicenseDetails;
import org.artifactory.addon.license.ArtifactoryHaLicenseDetails;
import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.api.subscriptions.SubscriptionType;
import org.artifactory.rest.common.model.artifactorylicense.BaseLicenseDetails;
import org.artifactory.rest.common.model.artifactorylicense.LicensesDetails;
import org.artifactory.rest.resource.system.providers.SubscriptionProviders;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import javax.ws.rs.core.Response;
import java.util.Collections;

/**
 * @author Alexei Vainshtein
 */
public class ArtifactoryLicensesResourceTest extends ArtifactoryHomeBoundTest {

    @Mock
    private ArtifactoryContext context;

    private AddonsManager addonsManager;

    @BeforeMethod
    private void setup() {
        MockitoAnnotations.initMocks(this);
        addonsManager = Mockito.mock(AddonsManager.class);
        Mockito.when(context.beanForType(AddonsManager.class)).thenReturn(addonsManager);
        ArtifactoryContextThreadBinder.bind(context);
    }

    @AfterMethod
    private void cleanup() {
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test(dataProviderClass = SubscriptionProviders.class, dataProvider = "subscriptionProvider")
    public void getLicenseInfoPro(SubscriptionType type) {
        ArtifactoryBaseLicenseDetails baseLicenseDetails =
                new ArtifactoryBaseLicenseDetails("", "", "", type);
        Mockito.when(addonsManager.getProAndAolLicenseDetails()).thenReturn(baseLicenseDetails);
        ArtifactoryLicensesResource artifactoryLicensesResource = new ArtifactoryLicensesResource();
        Response licenseInfo = artifactoryLicensesResource.getLicenseInfo();
        Assert.assertEquals(licenseInfo.getStatus(), HttpStatus.SC_OK);
        BaseLicenseDetails entity = (BaseLicenseDetails) licenseInfo.getEntity();
        Assert.assertEquals(entity.getSubscriptionType(), type.getString());
    }

    @Test
    public void getLicenseInfoHa() {
        homeStub.getHaNodeProperties().setProperty("enabled", "true");
        try {
            ArtifactoryHaLicenseDetails artifactoryHaLicenseDetails
                    = new ArtifactoryHaLicenseDetails("", "", "", "", false, "", "");
            Mockito.when(addonsManager.getClusterLicensesDetails())
                    .thenReturn(Collections.singletonList(artifactoryHaLicenseDetails));
            ArtifactoryLicensesResource artifactoryLicensesResource = new ArtifactoryLicensesResource();
            Response licenseInfo = artifactoryLicensesResource.getLicenseInfo();
            Assert.assertEquals(licenseInfo.getStatus(), HttpStatus.SC_OK);
            LicensesDetails entity = (LicensesDetails) licenseInfo.getEntity();
            entity.getLicenses().forEach(licenseFullDetails ->
                    Assert.assertNull(licenseFullDetails.getSubscriptionType()));
        } finally {
            homeStub.getHaNodeProperties().setProperty("enabled", "false");
        }

    }
}